/*-----------------------------------------------------------------------*
 Project: [HWR_GuiSample]
 File: [CtrlGraphics.h]

 Copyright (C) 2011 Nintendo Co., Ltd. All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
*-----------------------------------------------------------------------*/
//   `摀NX
// ----------------------------------------------------------------------
// Ql
//   C:\work\CTR\SampleDemos\demo1\graphics.cpp/h
// ----------------------------------------------------------------------
#ifndef __CTRL_GRAPHICS__
#define __CTRL_GRAPHICS__

// ----------------------------------------------------------------------
// include system header
// ----------------------------------------------------------------------
#include    "demo.h"

#include    <nn/gx.h>
#include    <nn/math.h>
#include    <nn/os.h>
#include    <nn/fnd.h>
#include    <nn/fs.h>
#include    <nn/y2r.h>

// ----------------------------------------------------------------------
// namespace
// ----------------------------------------------------------------------
using namespace nn::math;
using namespace nn::fnd;
using namespace demo;
using namespace nn::y2r;

// Rectangle structure
struct Rect
{
    int x;
    int y;
    int width;
    int height;
};

// Color structure
struct Color
{
    float red;
    float green;
    float blue;
};

// ----------------------------------------------------------------------
// class
// ----------------------------------------------------------------------
class CCtrlGraphics
{
public:
    CCtrlGraphics();
    ~CCtrlGraphics();

    u32         InitializeGraphics    ();
    void        FinalizeGraphics    ();
    void        Clear            (const s32 display );
    void        Swap            (const s32 display );
    void        SetColor        (const s32 display, GLfloat r, GLfloat g, GLfloat b);
    void        DrawLine        (const s32 display, f32 sx, f32 sy, f32 ex, f32 ey, bool _start);
    void        DrawRect        (const s32 display, f32 sx, f32 sy, f32 ex, f32 ey);
    void        DrawFull        (const s32 display, f32 sx, f32 sy, f32 ex, f32 ey);
    void        DrawCamera        (u8* textureDataPtr, u32 textureWidth, u32 textureHeight, u32 bmpWidth, u32 bmpHeight, u32 quality);
    void        DrawDateTime    (void);
    void        DrawText        (u32 x, u32 y, char* fmt, ...);
    void        DrawTextEx        (const s32 display, u32 x, u32 y, char* fmt, ...);
    u8*         GetTextureDataFromBmpFileData(const u32& bmpWidth, const u32& bmpHeight, u8* bmpDataBuffer, 
                                              u32& textureWidth, u32& textureHeight, u32 quality);
    void        WaitVsync        (const s32 display );
    bool        CheckTouchedInputBox( int _x, int _y) const;
    const        Rect* GetInputBoxRect() const;
    void         DrawInputBox    (void);
    void        SetDisplay        ( const s32 display );
protected:

private:
    void        InitTextureBuffer    (u32 textureWidth, u32 textureHeight);

    
    GLuint      m_PgID;         // program id
    GLuint      m_ShID;         // shader id
    uptr        m_HeapForGx;    // ExpHeap for app.

    demo::RenderSystemDrawing m_RenderSystemDrawing;

    GLuint      m_textureWidth;
    GLuint      m_textureHeight;
    u8*         m_textureGLDataBuffer;
    u8*         m_textureDataBuffer;

    static const int    mc_box_width  = 256;        // g̃ftHg
    static const int    mc_box_height = 256;        // g̃ftHgc
    Rect                m_input_box;                // ͘g
};

#endif    // __CTRL_GRAPHICS__
