/*-----------------------------------------------------------------------*
 Project: [HWR_GuiSample]
 File: [hwr_font.cpp]

 Copyright (C) 2011 Nintendo Co., Ltd. All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
*-----------------------------------------------------------------------*/
/******************************************************************
                        include t@C
*******************************************************************/
#include <nn/os.h>
#include <nn/fs.h>
#include <nn/font.h>
#include <nn/pl.h>
#include <nn/util.h>
#include "hwr_font.h"
#include "CTR_Define.h"

using namespace nn::os;
using namespace nn::fs;
using namespace nn::font;


/******************************************************************
            OJ֐
*******************************************************************/
/*!--------------------------------------------------------*
 @brief  : VF[_̏s܂
 @param  : pResource `p\[XǗIuWFNgւ̃|C^
 @return : `惊\[Ẍւ̃|C^
*----------------------------------------------------------*/
void*
InitShaders(nn::font::RectDrawer* pDrawer)
{
    const wchar_t* shaders = L"rom:/shbin/nwfont_RectDrawerShader.shbin";

    nn::fs::FileReader shaderReader(shaders);

    const u32 fileSize = (u32)shaderReader.GetSize();

    void* shaderBinary = OS_MALLOC(fileSize);
    NN_NULL_ASSERT(shaderBinary);

#ifndef NN_BUILD_RELEASE
    s32 read =
#endif // NN_BUILD_RELEASE
    shaderReader.Read(shaderBinary, fileSize);
    NN_ASSERT(read == fileSize);

    const u32 vtxBufCmdBufSize =
        nn::font::RectDrawer::GetVertexBufferCommandBufferSize(shaderBinary, fileSize);
    void *const vtxBufCmdBuf = OS_MALLOC(vtxBufCmdBufSize);
    NN_NULL_ASSERT(vtxBufCmdBuf);
    pDrawer->Initialize(vtxBufCmdBuf, shaderBinary, fileSize);

    OS_FREE(shaderBinary);

    return vtxBufCmdBuf;

}

/*!--------------------------------------------------------*
 @brief  : `̏ݒs܂
 @param  : width       ʂ̕
 @param  : height      ʂ̍
 @return : Ȃ
*----------------------------------------------------------*/
void
InitDraw(
    int     width,
    int     height
)
{
    // J[obt@
    // LCĎɍ킹āAƍւĂ܂
    const nn::font::ColorBufferInfo colBufInfo = { width, height, PICA_DATA_DEPTH24_STENCIL8_EXT };

    const u32 screenSettingCommands[] =
    {

        // r[|[g̐ݒ
        NN_FONT_CMD_SET_VIEWPORT( 0, 0, colBufInfo.width, colBufInfo.height ),

        // VU[𖳌
        NN_FONT_CMD_SET_DISABLE_SCISSOR( colBufInfo ),

        // wobt@̖
        // fvXW̐ݒ
        // |SItZbg̖
        NN_FONT_CMD_SET_WBUFFER_DEPTHRANGE_POLYGONOFFSET(
            0.0f,           // wScale : 0.0 Wobt@
            0.0f,           // depth range near
            1.0f,           // depth range far
            0,              // polygon offset units : 0.0  |SItZbg
            colBufInfo),
    };

    nngxAdd3DCommand(screenSettingCommands, sizeof(screenSettingCommands), true);

    static const u32 s_InitCommands[] =
    {
        // JO𖳌
        NN_FONT_CMD_SET_CULL_FACE( NN_FONT_CMD_CULL_FACE_DISABLE ),

        // XeVeXg𖳌
        NN_FONT_CMD_SET_DISABLE_STENCIL_TEST(),

        // fvXeXg𖳌
        // J[obt@̑SĂ̐݉
        NN_FONT_CMD_SET_DEPTH_FUNC_COLOR_MASK(
            false,  // isDepthTestEnabled
            0,      // depthFunc
            true,   // depthMask
            true,   // red
            true,   // green
            true,   // blue
            true),  // alpha

        // A[[fvXeXg𖳌
        NN_FONT_CMD_SET_ENABLE_EARLY_DEPTH_TEST( false ),

        // t[obt@ANZX
        NN_FONT_CMD_SET_FBACCESS(
            true,   // colorRead
            true,   // colorWrite
            false,  // depthRead
            false,  // depthWrite
            false,  // stencilRead
            false), // stencilWrite
    };

    nngxAdd3DCommand(s_InitCommands, sizeof(s_InitCommands), true);
}

/*!--------------------------------------------------------*
 @brief  : `̐ݒɖ߂܂
 @param  : Ȃ
 @return : Ȃ
*----------------------------------------------------------*/
void
ResetDraw(
   void
)
{
    u64 cmd[] = {
            0x0002025300000000,
            0x0002022900000000
    };
    nngxAdd3DCommand(cmd, sizeof(cmd), GL_TRUE);
    nngxUpdateState(NN_GX_STATE_ALL);
}
/*!--------------------------------------------------------*
 @brief  : LtHgǂݍޏ
 @param  : pFont    \ztHgւ̃|C^
 @return : LtHg̍\z̐
*----------------------------------------------------------*/
bool
InitFont(
    nn::font::ResFont*  pFont
)
{
    // LtHg̏
    NN_UTIL_PANIC_IF_FAILED(nn::pl::InitializeSharedFont());

    // LtHg̃[h܂őҋ@
    while (nn::pl::GetSharedFontLoadState() != nn::pl::SHARED_FONT_LOAD_STATE_LOADED)
    {
        // LtHg̃[hɎsĂȂmF
        if (nn::pl::GetSharedFontLoadState() == nn::pl::SHARED_FONT_LOAD_STATE_FAILED)
        {
            NN_TPANIC_("failed to load shared font!\n");
        }
        nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMilliSeconds(1));
    }

    // LtHgf[^̃AhX擾
    void* pFontBuffer = nn::pl::GetSharedFontAddress();

    // tHg\[XZbg܂
    bool bSuccess = pFont->SetResource(pFontBuffer);

    // `pobt@ݒ肵܂
    const u32 drawBufferSize = nn::font::ResFont::GetDrawBufferSize(pFontBuffer);
    void* drawBuffer = OS_MALLOC_EX(drawBufferSize, 4);
    pFont->SetDrawBuffer(drawBuffer);
    NN_NULL_ASSERT(drawBuffer);

    return bSuccess;
}

/*!--------------------------------------------------------*
 @brief  : `pobt@̖
 @param  : pFont    jtHgւ̃|C^
 @return : Ȃ
*----------------------------------------------------------*/
void
CleanupFont(nn::font::ResFont* pFont)
{
    // `pobt@̖
    // `pobt@ZbgĂȂ \z SetDrawBuffer ɓnobt@ւ
    // |C^ԂĂ܂
    void *const drawBuffer = pFont->SetDrawBuffer(NULL);
    if (drawBuffer != NULL)
    {
        OS_FREE(drawBuffer);
    }
}

/*!--------------------------------------------------------*
 @brief  : \pobt@mۂ
 @param  : charMax         \镶̍ő啶
 @return : mۂ\pobt@ւ̃|C^
*----------------------------------------------------------*/
nn::font::DispStringBuffer*
AllocDispStringBuffer(int charMax)
{
    const u32 DrawBufferSize = nn::font::CharWriter::GetDispStringBufferSize(charMax);
    void *const bufMem = OS_MALLOC(DrawBufferSize);
    NN_NULL_ASSERT(bufMem);

    return nn::font::CharWriter::InitDispStringBuffer(bufMem, charMax);
}


/*!--------------------------------------------------------*
 @brief  : \pɃfr[sƎˉesݒ肵܂
 @param  : pDrawer         RectDrawerIuWFNgւ̃|C^
 @param  : width           ʂ̕
 @param  : height          ʂ̍
 @return : Ȃ
*----------------------------------------------------------*/
void
SetupTextCamera(
    nn::font::RectDrawer*   pDrawer,
    int                     width,
    int                     height
)
{
    // ˉes𐳎ˉeɐݒ
    {
        // ㌴_ƂAYŽtɂȂ悤ɐݒ肵܂
        nn::math::MTX44 proj;
        f32 znear   =  0.0f;
        f32 zfar    = -1.0f;
        f32 t       =  0;
        f32 b       =  static_cast<f32>(width);
        f32 l       =  0;
        f32 r       =  static_cast<f32>(height);
        nn::math::MTX44OrthoPivot(&proj, l, r, b, t, znear, zfar, nn::math::PIVOT_UPSIDE_TO_TOP);
        pDrawer->SetProjectionMtx(proj);
    }

    // fr[sPʍsɐݒ
    {
        nn::math::MTX34 mv;
        nn::math::MTX34Identity(&mv);
        pDrawer->SetViewMtxForText(mv);
    }
}

/*!--------------------------------------------------------*
 @brief  : `悵܂(R[h)
 @param  : pDrawer         RectDrawerIuWFNgւ̃|C^
 @param  : pDrawStringBuf  DispStringBufferIuWFNgւ̃|C^
 @param  : pFont           tHgւ̃|C^
 @param  : pList           \镶R[hXg
 @param  : num             \镶R[hXg
 @return : Ȃ
*----------------------------------------------------------*/
void
DrawTextWithFontResult(
    nn::font::RectDrawer*           pDrawer,
    nn::font::DispStringBuffer*     pDrawStringBuf,
    nn::font::ResFont*              pFont,
    CODE_SIZE*                      pList,
    int                             num
)
{
    int cnt;
    f32 ypos=0.0f;
    char buf[16];
    nn::font::TextWriter writer;
    writer.SetDispStringBuffer(pDrawStringBuf);
    writer.SetFont(pFont);

    writer.SetScale(0.8f);

    pDrawer->DrawBegin();


    writer.SetWidthLimit(200.0);
    SetupTextCamera(pDrawer, NN_GX_DISPLAY0_WIDTH, NN_GX_DISPLAY0_HEIGHT);

    for( cnt=0;cnt<num;cnt++){
        writer.SetCursor(0, ypos);
        writer.StartPrint();
        (void)writer.Print(pList[cnt]);
        MEMSET(buf,0,sizeof(buf));
        sprintf( buf," (U-%04x)\n", pList[cnt]);
        (void)writer.Printf(buf);
        writer.EndPrint();
        pDrawer->BuildTextCommand(&writer);
        writer.UseCommandBuffer();
        ypos+=writer.GetFontHeight();
    }

    pDrawer->DrawEnd();
}

/*!--------------------------------------------------------*
 @brief  : `悵܂
 @param  : pDrawer         RectDrawerIuWFNgւ̃|C^
 @param  : pDrawStringBuf  DispStringBufferIuWFNgւ̃|C^
 @param  : pFont           tHgւ̃|C^
 @param  : pString         \Ώۂ̕
 @return : Ȃ
*----------------------------------------------------------*/
void
DrawTextWithFont(
    nn::font::RectDrawer*           pDrawer,
    nn::font::DispStringBuffer*     pDrawStringBuf,
    nn::font::ResFont*              pFont,
    char*                           pString
)
{
    nn::font::TextWriter writer;
    nn::util::Color8 color( nn::util::Color8::RED );
    writer.SetDispStringBuffer(pDrawStringBuf);
    writer.SetFont(pFont);

    writer.SetCursor(0, 0);
    writer.SetTextColor( color );

    pDrawer->DrawBegin();

    SetupTextCamera(pDrawer, NN_GX_DISPLAY0_WIDTH, NN_GX_DISPLAY0_HEIGHT);

    writer.StartPrint();
    (void)writer.Printf(pString);
    writer.EndPrint();
    pDrawer->BuildTextCommand(&writer);
    writer.UseCommandBuffer();

    pDrawer->DrawEnd();
}
/*!--------------------------------------------------------*
 @brief  : `悵܂
 @param  : pDrawer         RectDrawerIuWFNgւ̃|C^
 @param  : pDrawStringBuf  DispStringBufferIuWFNgւ̃|C^
 @param  : pFont           tHgւ̃|C^
 @param  : pString         \Ώۂ̕
 @param  : width           ʂ̕
 @param  : height          ʂ̍
 @return : Ȃ
*----------------------------------------------------------*/
void
DrawText(
    nn::font::RectDrawer*           pDrawer,
    nn::font::DispStringBuffer*     pDrawStringBuf,
    nn::font::ResFont*              pFont,
    char*                           pString,
    int                             x,
    int                             y
)
{
    nn::font::TextWriter writer;
    nn::util::Color8 color( nn::util::Color8::GREEN );
    writer.SetDispStringBuffer(pDrawStringBuf);
    writer.SetFont(pFont);

    writer.SetCursor(x, y);
    writer.SetTextColor( color );

    pDrawer->DrawBegin();

        SetupTextCamera(pDrawer, NN_GX_DISPLAY0_WIDTH, NN_GX_DISPLAY0_HEIGHT);

        writer.StartPrint();
        (void)writer.Printf(pString);
        writer.EndPrint();
        pDrawer->BuildTextCommand(&writer);
        writer.UseCommandBuffer();

    pDrawer->DrawEnd();
}
/*---------------------------------------------------------------------------*
  End of file
 *---------------------------------------------------------------------------*/

