/*-----------------------------------------------------------------------*
 Project: [HWR_GuiSample]
 File: [hwr_mw.cpp]

 Copyright (C) 2011 Nintendo Co., Ltd. All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
*-----------------------------------------------------------------------*/
/******************************************************************
                        include t@C
*******************************************************************/
#include "hwr_mw.h"

/* t@CpX */
#define DIC_FILE_PATH  TEST_DATA_ROOT"dic/"

/* Ǘe[u */
static const DicInfo g_DicName[DIC_NUM_MAX]= {
    {"all2_u"          ,DIC_FILE_PATH"all2_u.bin"          }, /* SQ           */
    {"all_u"           ,DIC_FILE_PATH"all_u.bin"           }, /* S             */
    {"alnum_u"         ,DIC_FILE_PATH"alnum_u.bin"         }, /* p             */
    {"alnum_native_u"  ,DIC_FILE_PATH"alnum_native_u.bin"  }, /* p(Native)     */
    {"alnumlat1_u"     ,DIC_FILE_PATH"alnumlat1_u.bin"     }, /* pe1      */
    {"alnumlat1_un"    ,DIC_FILE_PATH"alnumlat1_un.bin"    }, /* [}jA       */
    {"dutch_u"         ,DIC_FILE_PATH"dutch_u.bin"         }, /* I_         */
    {"french_u"        ,DIC_FILE_PATH"french_u.bin"        }, /* tX         */
    {"german_u"        ,DIC_FILE_PATH"german_u.bin"        }, /* hCc           */
    {"hangul_u"        ,DIC_FILE_PATH"hangul_u.bin"        }, /* nO         */
    {"hira_u"          ,DIC_FILE_PATH"hira_u.bin"          }, /* Ђ炪         */
    {"hirakataalnum_u" ,DIC_FILE_PATH"hirakataalnum_u.bin" }, /* ЂJ^p     */
    {"italian_u"       ,DIC_FILE_PATH"italian_u.bin"       }, /* C^A         */
    {"kanji2_u"        ,DIC_FILE_PATH"kanji2_u.bin"        }, /* Q           */
    {"kanji_u"         ,DIC_FILE_PATH"kanji_u.bin"         }, /* P           */
    {"kata_u"          ,DIC_FILE_PATH"kata_u.bin"          }, /* J^Ji         */
    {"korean_u"        ,DIC_FILE_PATH"korean_u.bin"        }, /* ؍           */
    {"korean_un"       ,DIC_FILE_PATH"korean_un.bin"       }, /* ؍(؍) */
    {"portuguese_u"    ,DIC_FILE_PATH"portuguese_u.bin"    }, /* |gK       */
    {"spanish_u"       ,DIC_FILE_PATH"spanish_u.bin"       }  /* XyC         */
};

/* F[hǗe[u */
static const ModeInfo g_ModeType[MODE_NUM_MAX]= {
    {RMODE_ALL     ,0, "ALL"     },   /* SF                   */
    {RMODE_SYMBOL  ,0, "SYMBOL"  },   /* L                                 */
    {RMODE_NUMBER  ,0, "NUMBER"  },   /*           OPQRSTUVWX   */
    {RMODE_UALPHA  ,0, "UALPHA"  },   /* p啶      ``y                 */
    {RMODE_LALPHA  ,0, "LALPHA"  },   /* p      `                 */
    {RMODE_HIRA    ,0, "HIRA"    },   /* Ђ炪      `       */
    {RMODE_KATA    ,0, "KATA"    },   /* J^Ji      @A` */
    {RMODE_GREEK   ,0, "GREEK"   },   /* MV                         */
    {RMODE_HANGUL  ,0, "HANGUL"  },   /* nO    *//* VA 啶Ɠl */
    {RMODE_HANJA   ,0, "HANJA"   },   /* ؍    *//* VA Ɠl */
    {RMODE_KANJI1  ,0, "KANJI1"  },   /* P    B0A1-CFD3            */
    {RMODE_KANJI2  ,0, "KANJI2"  }    /* Q    D0A1-F4A6            */
};


/*======================================================
 * O[oϐ Start
 *======================================================*/
static bool g_MW_InitFlag = false;

/* f[^WJpobt@ */
unsigned char*    g_DicBuff=NULL;

/* MՃf[^ */
mw::hwr::HwrRecgData        g_RecgData;

/*======================================================
 * O[oϐ End
 *======================================================*/


/*!--------------------------------------------------------*
 @brief  : getDicName()
 @param  : index  e[uQƗpIndex
 @return : 
 @summary: 擾
*----------------------------------------------------------*/
char* getDicName(int dicIdx){
    if( dicIdx < 0 || dicIdx > DIC_NUM_MAX ){
        PRINT_OUT("    [Error] dicNum Error\n");
        return NULL;
    }

    return g_DicName[dicIdx].name;
}

/*!--------------------------------------------------------*
 @brief  : getModeName()
 @param  : index  e[uQƗpIndex
 @return : F[h
 @summary: F[h擾
*----------------------------------------------------------*/
char* getModeName(int modeIdx){
    if( modeIdx < 0 || modeIdx > MODE_NUM_MAX ){
        PRINT_OUT("    [Error] modeNum Error\n");
        return NULL;
    }
    return g_ModeType[modeIdx].name;
}

/*!--------------------------------------------------------*
 @brief  : getModeType()
 @param  : index  e[uQƗpIndex
 @return : F[h
 @summary: F[h擾
*----------------------------------------------------------*/
short getModeType(int modeIdx){

    if( modeIdx < 0 || modeIdx > MODE_NUM_MAX ){
        PRINT_OUT("    [Error] modeNum Error\n");
        return -1;
    }
    return g_ModeType[modeIdx].type;
}

/*!--------------------------------------------------------*
 @brief  : getScore()
 @param  : index  e[uQƗpIndex
 @return : F[h
 @summary: F[h擾
*----------------------------------------------------------*/
short getScore(ResultInfo* result, int index) {

    if( index < 0 || index > DLL_RECV_CANDI ){
        return -1;
    }

    result->moji = g_RecgData.moji[index];
    result->deg  = g_RecgData.deg[index];

    return 0;
}

/*!--------------------------------------------------------*
 @brief  : hwr_mw_init()
 @param  : dicIdx   e[uQƗpIndexiǗj
 @param  : modeIdx  e[uQƗpIndexiF[hǗj
 @return :  ( 0)
         : s (-1)
*----------------------------------------------------------*/
short hwr_mw_init( int dicIdx, int modeIdx )
{
    short  ret=0;
    short  libRet;
    FILE*  dicFp=NULL;
    long   dicDataSize=0;
    long   readSize=0;
    int    modeType;

#ifdef ENABLE_TIMESCALE    
    OS_TICK             tx[2]={0,0};
#endif /* #ifdef ENABLE_TIMESCALE */

#ifdef ENABLE_TIMESCALE    
    tx[0]=tx[1]; // warningΉ
#endif /* #ifdef ENABLE_TIMESCALE */

    if( dicIdx < 0 || dicIdx > DIC_NUM_MAX ){
        PRINT_OUT("    [Error] dicNum Error\n");
        return -1;
    }

    if( modeIdx < 0 || modeIdx > MODE_NUM_MAX ){
        PRINT_OUT("    [Error] modeNum Error\n");
        return -1;
    }

    if( g_MW_InitFlag ){
        hwr_mw_end();
    }

    /*---------------------------------
     * O[oϐ
     *---------------------------------*/
    if( g_DicBuff ){
        OS_FREE( g_DicBuff );
        g_DicBuff=NULL;
    }
    g_DicBuff = NULL;                                /* f[^WJobt@ */
    memset( &g_RecgData, 0, sizeof(g_RecgData) );    /* mw::hwr::Execute()ƂI/Ff[^   */


    /*---------------------------------
     * f[^WJ
     *---------------------------------*/
    /* t@CI[v */
    dicFp = FILE_OPEN( g_DicName[dicIdx].fileName, "rb" );
    if( dicFp == NULL ){
        PRINT_OUT("    [Error] %s dicfile Open Error.\n", g_DicName[dicIdx].fileName );
        return -1;
    }
    
    /* t@CTCY擾 */
    dicDataSize = FILE_GETSIZE( dicFp );
    
    /* f[^WJpobt@̎擾 */
    g_DicBuff = (unsigned char*)OS_MALLOC32( dicDataSize );
    if( g_DicBuff == NULL ){
        PRINT_OUT("    [Error] %s dic buffer alloc Error\n", g_DicName[dicIdx].name );
        ret = -1;
        goto INIT_END;
    }
    
    /* f[^WJ */
    readSize = (long)FILE_READ( g_DicBuff, 1, dicDataSize , dicFp );
    if( readSize != dicDataSize ){
        PRINT_OUT("    [Error] %s dicfile Read Error.\n", g_DicName[dicIdx].fileName );
        ret = -1;
        goto INIT_END;
    }
    
    /* t@CN[Y */
    if( dicFp ){
        FILE_CLOSE( dicFp );
        dicFp = NULL;
    }

    /*---------------------------------
     * HWRCu
     *---------------------------------*/
#ifdef ENABLE_TIMESCALE
tx[0] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */    
    libRet = mw::hwr::Initialize( HWR_WAKU_W, HWR_WAKU_H, g_DicBuff );
#ifdef ENABLE_TIMESCALE
tx[1] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */
    PRINT_OUT("### mw::hwr::Initialize ###\n");

#ifdef ENABLE_TIMESCALE
    PRINT_OUT("    mw::hwr::Initialize() : %ld %s ", OS_TICKS_TO_MSEC(tx[1]-tx[0]), TIME_UNIT);
#else
    PRINT_OUT("    mw::hwr::Initialize() : ");
#endif /* #ifdef ENABLE_TIMESCALE */

    if( libRet == RT_OK ){
        PRINT_OUT("    yOKz\n");
    } else {
        ret = -1;
        PRINT_OUT("    yNGzret = %d\n",libRet);
    }
    

    /*---------------------------------
     * HWRCuF[hݒ
     *---------------------------------*/
    modeType = g_ModeType[modeIdx].type;


#ifdef ENABLE_TIMESCALE
tx[0] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */
    libRet = mw::hwr::ChangeMode(modeType);
#ifdef ENABLE_TIMESCALE
tx[1] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */

    PRINT_OUT("### mw::hwr::ChangeMode (0x%04x)###\n", modeType);
#ifdef ENABLE_TIMESCALE
    PRINT_OUT("    mw::hwr::ChangeMode() : %ld %s ", OS_TICKS_TO_MSEC(tx[1]-tx[0]), TIME_UNIT);
#else
    PRINT_OUT("    mw::hwr::ChangeMode() : ");
#endif /* #ifdef ENABLE_TIMESCALE */

    if( libRet == RT_OK ) {
        PRINT_OUT("    yOKz\n");
        ret = 0;
    } else {
        PRINT_OUT("    yNGzret = %d\n",libRet);
        ret = -1;
        goto INIT_END;
    }

    g_MW_InitFlag = true;
    return ret;

INIT_END:
    if( g_DicBuff ){
        OS_FREE( g_DicBuff );
        g_DicBuff = NULL;
    }
    if( dicFp ){
        FILE_CLOSE( dicFp );
        dicFp = NULL;
    }    

    g_MW_InitFlag = false;
    return ret;

}

/*!--------------------------------------------------------*
 @brief  : hwr_mw_exec()
 @param  : position   ͍W_
 @return :  (Fʐ)
         : s (-1)
*----------------------------------------------------------*/
short hwr_mw_exec( u8* position, long len )
{
    short ret=0;
    s16 libRet;
    int i;

#ifdef ENABLE_TIMESCALE    
    OS_TICK             tx[2]={0,0};
#endif /* #ifdef ENABLE_TIMESCALE */

    if( position == NULL ){
        PRINT_OUT("    [Error] position = NULL \n");
    }

    if( len <= 0 || len > IDAT_MAX_POINT ){
        PRINT_OUT("    [Error] len = %d\n");
        return -1;
    }

    if( g_MW_InitFlag == false ){
        PRINT_OUT("    [Error]StatusNG\n");
        ret = -1;
        goto EXEC_END;
    }

#ifdef ENABLE_TIMESCALE
    tx[0]=tx[1]; // warningΉ
#endif /* #ifdef ENABLE_TIMESCALE */

    /*---------------------------------
     * HWRCuF
     *---------------------------------*/
    PRINT_OUT("\n");
    //MՃf[^ݒ
    for( i=0;i<len;i++){
        g_RecgData.idat[i*2]  = (u8)( position[i*2]);
        g_RecgData.idat[i*2+1]= (u8)( position[i*2+1]);
    }
    
    /* I_f[^ݒ */
    g_RecgData.idat[i*2]=IDAT_TERM_POINT;
    g_RecgData.idat[i*2+1]=IDAT_TERM_POINT;
    
#ifdef ENABLE_TIMESCALE
tx[0] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */

    libRet = mw::hwr::Execute( &g_RecgData );
#ifdef ENABLE_TIMESCALE
tx[1] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */

    PRINT_OUT("### mw::hwr::Execute ###\n");
#ifdef ENABLE_TIMESCALE
    PRINT_OUT("    mw::hwr::Execute() : %ld %s ", OS_TICKS_TO_MSEC(tx[1]-tx[0]), TIME_UNIT);
#else
    PRINT_OUT("    mw::hwr::Execute() : ");
#endif /* #ifdef ENABLE_TIMESCALE */

    if( libRet == RT_NG ) {
        PRINT_OUT("    yNGzret = %d\n",libRet);
        ret = -1;
    }
    else if( libRet == 0 ){
        PRINT_OUT("    yOKzUnknown (%d) \n",libRet);
        ret = 0;
    } else {
        PRINT_OUT("    yOKzResult = %d \n",libRet);
        ret = 0;
        /* Fʏo */
        for( i = 0; i < libRet; i++ ) {
//            PRINT_OUT( "    Unicode:0x%04x\n", g_RecgData.moji[i] );
        }
        ret = libRet;
    }

    return ret;

EXEC_END:
    return ret;

}

/*!--------------------------------------------------------*
 @brief  : hwr_mw_end()
 @param  : Ȃ
 @return :  ( 0)
         : s (-1)
*----------------------------------------------------------*/
short hwr_mw_end( void )
{
     short ret=0;
     s16 libRet;
#ifdef ENABLE_TIMESCALE    
    OS_TICK             tx[2]={0,0};
#endif /* #ifdef ENABLE_TIMESCALE */

#ifdef ENABLE_TIMESCALE
    tx[0]=tx[1]; // warningΉ
#endif /* #ifdef ENABLE_TIMESCALE */


    if( g_MW_InitFlag == false ){
        PRINT_OUT("    [Error]StatusNG\n");
        ret = -1;
        // CuIĂȂ\̂
        // End()͔Ô߃R[
    }

    /*---------------------------------
     * HWRCuI
     *---------------------------------*/
#ifdef ENABLE_TIMESCALE
tx[0] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */
    libRet = mw::hwr::Finalize();
#ifdef ENABLE_TIMESCALE
tx[1] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */
    PRINT_OUT("### mw::hwr::Finalize ###\n");
#ifdef ENABLE_TIMESCALE
    PRINT_OUT("    mw::hwr::Finalize() : %ld %s ", OS_TICKS_TO_MSEC(tx[1]-tx[0]), TIME_UNIT);
#else
    PRINT_OUT("    mw::hwr::Finalize() : ");
#endif /* #ifdef ENABLE_TIMESCALE */


    if( libRet == RT_OK ){
        PRINT_OUT("    yOKz\n");
        ret = 0;
    } else {
        PRINT_OUT("    yNGzret = %d\n",libRet);
        ret = -1;
    }

    memset( &g_RecgData, 0, sizeof(g_RecgData) );    /* mw::hwr::Execute()ƂI/Ff[^   */

    /*---------------------------------
     * eI
     *---------------------------------*/
    if( g_DicBuff ){
        OS_FREE( g_DicBuff );
        g_DicBuff = NULL;
    }

    g_MW_InitFlag=false;
    return ret;

}



/*!--------------------------------------------------------*
 @brief  : hwr_mw_mode()
 @param  : modeIdx  e[uQƗpIndexiF[hǗj
 @return :  ( 0)
         : s (-1)
*----------------------------------------------------------*/
short hwr_mw_mode( int modeIdx )
{
    short  ret=0;
    short  libRet;
    int    modeType;

#ifdef ENABLE_TIMESCALE    
    OS_TICK             tx[2]={0,0};
#endif /* #ifdef ENABLE_TIMESCALE */

#ifdef ENABLE_TIMESCALE    
    tx[0]=tx[1]; // warningΉ
#endif /* #ifdef ENABLE_TIMESCALE */


    if( modeIdx < 0 || modeIdx > MODE_NUM_MAX ){
        PRINT_OUT("    [Error] modeNum Error\n");
        return -1;
    }

    if( g_MW_InitFlag == false ){
        PRINT_OUT("    [Error]StatusNG\n");
        return -1;
    }

    /*---------------------------------
     * HWRCuF[hݒ
     *---------------------------------*/
    modeType = g_ModeType[modeIdx].type;


#ifdef ENABLE_TIMESCALE
tx[0] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */
    libRet = mw::hwr::ChangeMode(modeType);
#ifdef ENABLE_TIMESCALE
tx[1] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */

    PRINT_OUT("### mw::hwr::ChangeMode (0x%04x)###\n", modeType);
#ifdef ENABLE_TIMESCALE
    PRINT_OUT("    mw::hwr::ChangeMode() : %ld %s ", OS_TICKS_TO_MSEC(tx[1]-tx[0]), TIME_UNIT);
#else
    PRINT_OUT("    mw::hwr::ChangeMode() : ");
#endif /* #ifdef ENABLE_TIMESCALE */

    if( libRet == RT_OK ) {
        PRINT_OUT("    yOKz\n");
        ret = 0;
    } else {
        PRINT_OUT("    yNGzret = %d\n",libRet);
        ret = -1;
    }

    return ret;

}

/*---------------------------------------------------------------------------*
  End of file
 *---------------------------------------------------------------------------*/

