/*-----------------------------------------------------------------------*
 Project: [Common]
 File: [CTR_Memory.cpp]

 Copyright (C) 2009 Nintendo Co., Ltd. All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
*-----------------------------------------------------------------------*/
/******************************************************************
                        include t@C
*******************************************************************/
#include "CTR_Memory.h"
#include "CTR_Define.h"
#include <nn/fnd.h>
#include <string.h> // memset
using namespace nn::fnd;

#ifdef __cplusplus
extern "C" {
#endif


/******************************************************************
            static 錾
*******************************************************************/
/* ExpHeap for app. */
static ExpHeap gAppHeap;
static int gAllocCount=0;

/******************************************************************
            OJ֐
*******************************************************************/
/*!--------------------------------------------------------*
 @brief  : CTR_MEM_Init()
 @param  : Ȃ
 @return :  0
           s -1
 @summary: ֘Ȁ
 @note   : O nn::fnd::Initilizse()sĂ邱
*----------------------------------------------------------*/
int CTR_MEM_Init(void)
{
    gAllocCount=0;
    // q[v̍쐬
#if 1 // tHg\pɊgB
    const s32 nDeviceMemorySizeOther = 4 * 1024 * 10240*4;
    gAppHeap.Initialize(nn::os::GetDeviceMemoryAddress(), nDeviceMemorySizeOther, nn::os::ALLOCATE_OPTION_LINEAR);
#else
    gAppHeap.Initialize(nn::os::GetDeviceMemoryAddress(), NN_OS_DEVICE_MEMORY_SIZE, nn::os::ALLOCATE_OPTION_LINEAR);
#endif

    PRINT_OUT("[Heap Rest = %ld allocCnt=%d @Init()\n", CTR_MEM_GetFreeSizeOfHeap(), gAllocCount);
 
    return 0;
}

/*!--------------------------------------------------------*
 @brief  : CTR_MEM_Term()
 @param  : Ȃ
 @return : Ȃ
 @summary: mۂ̈
*----------------------------------------------------------*/
void CTR_MEM_Term(void)
{
    PRINT_OUT("[Heap Rest = %ld allocCnt=%d @Term()\n", CTR_MEM_GetFreeSizeOfHeap(), gAllocCount);
    gAppHeap.Finalize();
    return ;
}

/*!--------------------------------------------------------*
 @brief  : CTR_MEM_Malloc32
 @param  : size   mۂTCY
 @return :  mۂ̈̃AhX
           s NULL
 @summary: m(32oCgAC)
 @note   : 32bytẽACg̏ꍇgp
*----------------------------------------------------------*/
void *CTR_MEM_Malloc32( size_t size )
{
    void *allocator = NULL;
    allocator = (gAppHeap.Allocate( size, 32 ));
	if( allocator == NULL ){
        PRINT_OUT(" [ERROR] Alloc Failed\n");
		return NULL;
	}
    MEMSET( allocator, 0, size );
    gAllocCount++;
#ifdef MEMORY_DEBUG
    PRINT_OUT("   Alloc Size=%d rest=%ld count=%d (0x%08x)\n", size, CTR_MEM_GetFreeSizeOfHeap(),gAllocCount, allocator);
#endif /* #ifdef MEMORY_DEBUG */
    return allocator;
}


/*!--------------------------------------------------------*
 @brief  : CTR_MEM_Malloc
 @param  : size   mۂTCY
 @return :  mۂ̈̃AhX
           s NULL
 @summary: m(4oCgAC)
 @note   : 32bytẽACg̏ꍇgp
*----------------------------------------------------------*/
void *CTR_MEM_Malloc( size_t size )
{
    void *allocator=NULL;
    allocator = (gAppHeap.Allocate(size));
	if( allocator == NULL ){
        PRINT_OUT(" [ERROR] Alloc Failed\n");
		return NULL;
	}
    MEMSET( allocator, 0, size );
    gAllocCount++;
#ifdef MEMORY_DEBUG
    PRINT_OUT("   Alloc Size=%d rest=%ld count=%d (0x%08x)\n", size, CTR_MEM_GetFreeSizeOfHeap(),gAllocCount, allocator);
#endif /* #ifdef MEMORY_DEBUG */
    return allocator;

}

/*!--------------------------------------------------------*
 @brief  : CTR_MEM_MallocEx
 @param  : size       mۂTCY
 @param  : alignment  ACgw
 @return :  mۂ̈̃AhX
           s NULL
 @summary: w肳ꂽACgŃmۂ
 @note   : alignment Ɏw\ȒĺA
           -128, -64, -32, -16, -8, -4, 4, 8, 16, 32, 64, 128
           ̂ꂩ̒lłB
*----------------------------------------------------------*/
void *CTR_MEM_MallocEx( size_t size, int alignment )
{
    void *allocator=NULL;
    allocator = (gAppHeap.Allocate(size, alignment));
	if( allocator == NULL ){
        PRINT_OUT(" [ERROR] Alloc Failed\n");
		return NULL;
	}
    MEMSET( allocator, 0, size );
    gAllocCount++;
#ifdef MEMORY_DEBUG
    PRINT_OUT("   Alloc Size=%d align(%d) rest=%ld count=%d (0x%08x)\n", size, alignment, CTR_MEM_GetFreeSizeOfHeap(),gAllocCount, allocator);
#endif /* #ifdef MEMORY_DEBUG */
    return allocator;

}

/*!--------------------------------------------------------*
 @brief  : CTR_MEM_Free()
 @param  : ptr     ̈̃AhX
 @return : Ȃ
 @summary: mۂ
*----------------------------------------------------------*/
void CTR_MEM_Free( void *ptr )
{
    gAppHeap.Free( ptr );
    gAllocCount--;

#ifdef MEMORY_DEBUG
    PRINT_OUT("   Free() rest=%ld count=%d (0x%08x)\n", CTR_MEM_GetFreeSizeOfHeap(),gAllocCount,ptr);
#endif /* #ifdef MEMORY_DEBUG */

    return;
}


/*!--------------------------------------------------------*
 @brief  : CTR_MEM_GetFreeSizeOfHeap()
 @param  : Ȃ
 @return : cq[v̈TCY
 @summary: q[v̈̎cTCY擾
*----------------------------------------------------------*/
unsigned long CTR_MEM_GetFreeSizeOfHeap( void )
{
    return (unsigned long)( gAppHeap.GetTotalFreeSize() );
}


/*!--------------------------------------------------------*
 @brief  : CTR_MEMSET()
 @param  : s  LΏ
 @param  : c  
 @param  : n  
 @return : LΏۂ̕ւ̃|C^
 @summary: memset()Ɠ
*----------------------------------------------------------*/
void *CTR_MEMSET( void* s, int c, size_t n )
{
    std::memset( s,c,n );
    return s;
}


/*!--------------------------------------------------------*
 @brief  : CTR_MEMCPY()
 @param  : s  Rs[
 @param  : c  Rs[
 @param  : n  Rs[oCg
 @return : Ȃ
 @summary: memcpy()Ɠ
*----------------------------------------------------------*/
void CTR_MEMCPY( void* dst, const void* src, size_t size )
{
    std::memcpy( dst, src, size );
    return;
}

/*!--------------------------------------------------------*
 @brief  : CTR_MEMCPY()
 @return : mem1 rΏۃAhX1
 @return : mem2 rΏۃAhX2
 @return : size rTCY 
 @summary: rΏۂȂ 0Amem1 Ȃ畉A
           mem1 傫Ȃ琳̒lԂ܂B
 @summary: memcmp()Ɠ
*----------------------------------------------------------*/
int CTR_MEMCMP( const void* mem1, const void* mem2, size_t size )
{
   return std::memcmp( mem1, mem2, size );
}

#ifdef __cplusplus
}
#endif



